import os
import os.path
import sys
import csv
import glob
import cgi
import codecs, cStringIO

__version__ = "1.01"

class UTF8Recoder:
    """
    Iterator that reads an encoded stream and reencodes the input to UTF-8
    """
    def __init__(self, f, encoding):
        self.reader = codecs.getreader(encoding)(f)

    def __iter__(self):
        return self

    def next(self):
        return self.reader.next().encode("utf-8")
        
class UnicodeReader:
    """
    A CSV reader which will iterate over lines in the CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
        f = UTF8Recoder(f, encoding)
        self.reader = csv.reader(f, dialect=dialect, **kwds)

    def next(self):
        row = self.reader.next()
        return [unicode(s, "utf-8") for s in row]

    def __iter__(self):
        return self


def	Check_For_Indigo_Log_Directory ():
	if os.path.exists (r'/Library/Application Support/Perceptive Automation/Indigo 4/Logs'):
		indigo_path = r'/Library/Application Support/Perceptive Automation/Indigo 4/Logs'
	else:
		indigo_path = r'/Library/Application Support/Perceptive Automation/Indigo 2/Logs'
	return indigo_path
	
#def unicode_csv_reader(unicode_csv_data, dialect=csv.excel, **kwargs):
#    # csv.py doesn't do Unicode; encode temporarily as UTF-8:
#    csv_reader = csv.reader(utf_8_encoder(unicode_csv_data),
#                            dialect=dialect, **kwargs)
#    for row in csv_reader:
#        # decode UTF-8 back to Unicode, cell by cell:
#        yield [unicode(cell, 'utf-8') for cell in row]##

#def utf_8_encoder(unicode_csv_data):
 #   for line in unicode_csv_data:
 #       yield line.encode('utf-8')
        
def read_indigo_log_file(filename, csv_data=[], tag_data=[], dates=[], filter=["",]):
	#
	#		Read in the Indigo Log file (FQPN in filename, and append the CSV, Tag, and dates to the Lists passed into the function
	#
	filename = filename.strip()

	#csv_file = csv.reader(open(filename, "rU"), delimiter = '\t')
	#csv_file = unicode_csv_reader ( open(filename, "rU"), delimiter = '\t')
	csv_file = UnicodeReader ( open(filename, "rU"), delimiter = "\t" )
	
	for log_line in csv_file:
		if len(log_line) == 3:
			timestamp = log_line[0];
			logtype = log_line[1];
			logdesc = log_line[2];
		elif len(log_line) == 1:
			timestamp = ""
			logtype = ""
			logdesc = log_line[0];
		logdesc = cgi.escape ( logdesc )

		if timestamp <> "" and logtype<> "":
			#
			#	Entry isn't empty, and there is a timestamp
			#
			if filter<>["",] and filter.count(logtype.strip().upper())==0:
				# The filter isn't an empty value, and the log entry does not met the filter criteria
				# then don't append the log entry to the csv_data pool
				pass
			else:
				#
				#	No filter, or does match filter criteria.  Add to CSV Pool
				csv_data.append ( (timestamp.strip(), logtype.strip(), logdesc.strip()) )

		if logtype != "" and tag_data.count (logtype.strip() ) == 0:
			#
			#	Valid Log entry, and the log tag has not been seen previously.  Add to tag Pool.
			#
			tag_data.append ( logtype.strip() )
			
		if timestamp != "" and dates.count ( timestamp.split(" ")[0]) == 0:
			#
			#	Valid log entry, and we have not seen this date yet.  Add to date pool.
			#
			dates.append ( timestamp.split(" ")[0] )

	if sys.version_info[:3] >= (2,4,0):
		tag_data.sort( reverse = True)
		csv_data.sort ( reverse = True)
	else:
		tag_data.reverse ()
		csv_data.reverse ()
		
	return (csv_data, tag_data, dates)
