####################
# Copyright (c) 2009, Perceptive Automation, LLC. All rights reserved.
# http://www.perceptiveautomation.com
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
####################

####################
## IMPORTS
import cherrypy
from cherrypy import _cperror

from indigopy import indigoconn as ic
from indigopy import indigodb as idb
from indigopy.basereqhandler import BaseRequestHandler, kTrueStr, kFalseStr, kEmptyStr, kTextPageStr, kHtmlPageStr, kXmlPageStr
#import cgitb; cgitb.enable()
#from custom_colors import *
import socket
import os
import os.path
import sys
import csv
import glob
import cgi
import Indigo_Python_Utils
####################
## CONSTANTS

####################
# Optional hooks to provide a plugin name and description (shown in Event Log and plugin index)
def PluginName():
	return u"Security Status Plugin"

def PluginDescription():
	return u"Displays recent Event Log files."

def ShowOnControlPageList():
	return True		# if True, then above name/description is shown on the Control Page list index

# Optional hook called when the IndigoWebServer first connects to IndigoServer
def IndigoConnected():
	pass

# Optional hook called when the IndigoWebServer disconnect from IndigoServer
def IndigoDisconnected():
	pass


####################
# The functions in this class are automatically called based on the URL
# requested. This provides a mechanism for easily serving dynamic content.
#
# The URL path is based on the folder path of the plugin. For example,
# this plugin folder name is "sample" so any URL path containing "sample"
# as the first path identifier will call into these functions.
#
# The class name can be called whatever you want, but it must subclass
# from BaseRequestHandler to be loaded by the webserver plugin manager.
class HaloHomeRequestHandler(BaseRequestHandler):
	""" Handles HTTP page requests. """

	####################
	# The plugin page request handler must subclass from BaseRequestHandler
	# and must call the parent class __init__ method.
	def __init__(self, logFunc, debugLogFunc):
		BaseRequestHandler.__init__(self, logFunc, debugLogFunc)


	def form_top ( self, html_elems):
		#
		#		Contains the top of the report
		#		Split from the index function to clarify / simplify the visual clutter
		#
		html_elems.append ("<head>\n")
		html_elems.append("<title>Indigo log viewer</title></Head>\n")
		html_elems.append('<link rel="stylesheet" type="text/css" href="css/EventLog.css">')
		html_elems.append ('<meta name="apple-mobile-web-app-capable" content="yes">')
		html_elems.append ('<meta name="viewport" content="width=320,initial-scale=0.8,maximum-scale=1.6,user-scalable=yes"/>')

		html_elems.append("<body>\n")
		html_elems.append ('''<script type="text/javascript" src="js/prototype.js"></script>
<script type="text/javascript" src="js/tablekit.js"></script>''')
		
		return html_elems
		
		
	####################
	def	return_variable (self, variable_name, variables):
		for x in variables:
			if variable_name.strip() == x.name.strip():
				return x.value
		return ""

	def	return_variables_with_security_not_from_sec_script ( self, variableList ):
		data = []
		for x in variableList:
			if not self.is_variable_from_security_script ( x ) and x.name.upper().find ("SECURITY") <> -1:
				data.append ( x )
		return data
		
	def	is_variable_from_security_script ( self, variable ):
		return (variable.name.find ("_is_open") <> -1  or
			variable.name.find ("_last_status") <> -1 or
			variable.name.find ("_last_update") <> -1)

	def find_security_data (self, Indigo_Variables):
		#
		#
		#
		security_info = {}
		for variable in Indigo_Variables:
			if variable.name.find ("_is_open")<>-1:
				base_name = str(variable.name[0:variable.name.rfind ("_")-3])
				last_status = self.return_variable (base_name+"_last_status", Indigo_Variables)
				last_update = self.return_variable (base_name+"_last_update", Indigo_Variables)
				security_info [base_name] = (last_update, last_status)
		return security_info


	def	return_security_log_entries ( Number_of_Lines = 25 ):
		"""
		"""
		html_page = []
		csv_data = []
		tag_data = []
		dates = []

		indigo_path = Indigo_Python_Utils.Check_For_Indigo_Log_Directory ()
				
		log_files = glob.glob (indigo_path+'/*')
			#
			#		Only load numdays worth of files, starting at the end of the list
			#
		if sys.version_info[:3] >= (2,4,0):
			log_files.sort( reverse=True)
		else:
			log_files.reverse ()

#		html_page.append ( "Log Files Length %i<br>" % len(log_files) )
		log_files = log_files[0:3] # Only load the last X log files (x = numofdays)
#		html_page.append ( "Log Files Length %i<br>" % len(log_files) )
		
		for filename in log_files:
			#
			#		Load each log file into the buffer
			#		Since we are dealing with a limited number of lines, limit the load to the last 3 days
#			html_page.append ( "Log File - %s<br>" % filename)
			csv_data, tag_data, dates = Indigo_Python_Utils.read_indigo_log_file(r'%s' % filename, csv_data, tag_data, dates, filter=["SECURITY",])

		
#		html_page.append ("CSV Data Length - %i" % len(csv_data) )
		html_page.append ("<center>")
		html_page.append ("<H2>The Last 25 Lines of Security Entries from Indigo's Log File</H2>")
		html_page.append ('	<table class="sortable resizable strip" border=1 cellpadding=1 cellspacing=0>')
		html_page.append (" <TR> <TD><CENTER><B>Time & Date</TD> <TD><CENTER><B>Class</TD><TD><CENTER><B>Log Entry</TD></TR>")
	
		line_count = 0
		for (timestamp, tag, text) in csv_data:
			#
			#	Bold font anything that is in quotes
			#
			if text.count ('"') >= 2:
					start_text = text.find ('"')
					end_text = text.find ('"', start_text+1)
					text_length = len(text)
					text = text[0:start_text] + "<B>" + text[start_text:end_text] + "</B>" + text[end_text:text_length]

			clean_text = text.strip()
			clean_tag = tag.strip()
			clean_tag_css = tag.strip().replace(" ","").replace("/", "").replace("(","").replace(")","")

			text_entry = '		 <TR><TD width="25%%" class="DateTimeStamp">%s</TD><TD width="15%%" align=center class="%s">%s</TD><TD class="%s">%s</TD></TR>\n' % (timestamp, clean_tag_css, tag, clean_tag_css, text)

			if type (text_entry) <> unicode:
				html_page.append( text_entry.decode ('utf-8'))
			else:
				html_page.append ( text_entry )
				  
			line_count = line_count + 1
			if line_count >= 25: break
		html_page.append("		</table></center><p><p>\n")
		return ''.join(html_page)

	####################
	def index( self ):
		#
		#
		variableList = cherrypy.server.indigoDb.GetVariables(cherrypy.server.indigoConn, True) 
		cherrypy.response.headers['Content-Type'] = 'text/html'
		html_elems = []
		html_elems = self.form_top ( html_elems)
		
		html_elems.append("<html>\n")

#		test = self.return_variable ( "sprinklerDurationMultiplier", variableList)
#		html_elems.append  ( repr( test ) )

		html_elems.append ("<center>")
		html_elems.append ("<H2>Security Sensor Status</H2>")
		html_elems.append ("<P>")
		html_elems.append ("The Last Heartbeat was processed on: %s<br>" % self.return_variable ( "Checked_Heartbeats", variableList) )
		html_elems.append ("The Chime Last Sounded at: %s<br>" % self.return_variable ( "Chime_Last_Sounded", variableList) )
		html_elems.append ("<P><hr>")
		html_elems.append ('<table class="sortable resizable strip" border=1 cellpadding=2 cellspacing=2>')
		html_elems.append (" <TR> <TD><CENTER><B>Sensor Name</TD> <TD><CENTER><B>Last Status</TD> <TD><CENTER><B>Last Updated</TD> </TR>")

		security_info = self.find_security_data ( variableList)

		for x in security_info.keys():
			html_elems.append ("<TR>")
			html_elems.append ("<TD>%s</TD><TD><center>%s</center></TD><TD>%s</TD>" % (x, security_info[x][1], security_info[x][0]) )
			html_elems.append ("</TR>")

		html_elems.append("	   </table>\n")
		html_elems.append ("</center>")
		html_elems.append ("<hr>")

		html_elems.append ("<center><H2>Variables that may be Security Related</H2>")
		html_elems.append ('<table class="sortable resizable strip" border=1 cellpadding=2 cellspacing=2>')
		html_elems.append (" <TR> <TD><CENTER><B>Variable Name</TD> <TD><CENTER><B>Data</TD></TR>")
		for x in self.return_variables_with_security_not_from_sec_script ( variableList ):
			html_elems.append ("<TR><TD>%s</TD><TD>%s</TD></TR>" % ( x.name.strip(), x.value) )
		html_elems.append ("</table><center>")
		html_elems.append ("<hr>")
		

		html_elems.append (self.return_security_log_entries () )

		html_elems.append ("<hr>")

# 		security_info = self.find_security_data ( variableList)
# 		html_elems.append ( repr(security_info) )
# 		html_elems.append ("</form>")
		html_elems.append("</body>\n")
		
		return ''.join(html_elems)
	index.exposed = True

# IsEditable
# StreamPublicKeyVals
# _GetPublicKeys
# __cmp__
# __doc__
# __init__
# __module__
# _editableAttrs
# _typeName
# displayInUI
# isFalse
# name
# readOnly
# value
# GetTypeName


#
#		Version History - 
#
